#include <Wire.h>
#include <HexCommands.h>               //Лист команд
#include <avr/wdt.h>
//#define DEBUG 1
//#define NO_CMD_TIME 500
//bool enginesStopped;//engine stopped flag
//uint32_t noCmdTimer;//timer used to send reply when no command sent
//added sep-2021
uint32_t enginesCmdTime;//time of last engines command
uint32_t enginesDelay;//time to wait before stop engines
byte lastCmd = 0;
bool replyEnabled = true;
//-----------------------------
//Main procedures
//-----------------------------
void setup()
{
#ifdef DEBUG
  Serial.begin(115200);
  Serial.println("begin Setup");
#endif
  //power monitoring first!
  SetupPowerMonitoring();
  SetupRadio();
  SetupVideo();
  SetupVTX();
  SetupMotors();
  SetupLamp();
  SetupDistanceSensor();
  //setup roboarm
  SetupRoboArm();
  setupBuzzer();
  DiagDataBag.Error |= errReady;
  SetupCompass();
  SetupFoots();
  Buzzer_KexReady();
  //TestHoba();
  //TestESCMotors();
  wdt_enable (WDTO_4S);//WatchDog enabled //WDTO_8S
#ifdef DEBUG
  //ReportPowerMon();
  Serial.println("End Setup");
#endif

}

void loop()
{
  //watchdog first!
  wdt_reset();
  //stopengines management
  if (enginesCmdTime > 0)
  {
    unsigned long currentTime = millis();
    if (enginesCmdTime > 0 &&
        currentTime > (enginesCmdTime + enginesDelay))
    {
      StopEngines();
      //reply не посылаем при forward/backward/rotate
      //а только при команде stopengines либо при срабатываниии
      //timeout работы движков
      SendReply();
      lastCmd = 0;//сбрасываем последнюю команду
#ifdef DEBUG
      Serial.println("StopEngines on TimeOut");
      Serial.print("Current:"); Serial.println(currentTime);
      Serial.print("CMD Time:"); Serial.println(enginesCmdTime);
#endif
#ifdef DEBUG
      Serial.println("Aghrh-StopEngines!");
#endif
    }
  }
  if (DataAvailable())// Если в буфере поступили данные
  {
    DiagDataBag.Error = errNoErrors;//clear errors
#ifdef DEBUG
    Serial.println("Data Available");
#endif
    ProcessCommand(DataBag.CmdBefore);
    ProcessCommand(DataBag.Command);
    ProcessCommand(DataBag.CmdAfter);
    lastCmd = DataBag.Command;//регистрируем последнюю команду
    CheckCurrentDC();
    if (ReplyRequired())//we'll send reply only in special cases - servo + mode buttons (line, lamp, learn)
      SendReply();//send current state back
  }
  else//if no command available-check overcurrent
  {
#ifndef DEBUG
    CheckCurrentDC();
#endif
  }
}

void ProcessCommand(byte command)
{
  //command from joystick - first byte
  switch (command)
  {
    //    case cmdScript1:
    //      AutoTurnOverTask();
    //      break;
    case cmdVTXChangeChannel:
      vtx_setChannel();
      break;
    case cmdVTXChangePowerLevel:
      vtx_setPower();
      break;
    case cmdHandShake://ничего не делает, только отсылает телеметрию
#ifdef DEBUG
      Serial.println(F("Handshake"));
#endif
      break;
    case cmdOverloadCheckOn:
      SetCheckCurrentDCMode(true);
      break;
    case cmdDistanceSensorEnable://start use distance sensor
      EnableDistanceMeasure(true);
      break;
    case cmdDistanceSensorDisable://end use of distance sensor
      EnableDistanceMeasure(false);
      break;
    case cmdDistanceSensorToggle://toggle use of distance sensor
      ToggleDistanceMeasure();
      break;
    case cmdOverloadCheckOff:
      SetCheckCurrentDCMode(false);
      break;
    case cmdMoveStepForward:
#ifdef DEBUG
      Serial.println("Step Forward");
#endif
      //commented - because can be serial cmd like cmdMove_froward
      //if (IsNotStepCommand(lastCmd))
      {
        StepMove(command);
        replyEnabled = false;
      }
      break;
    case cmdMoveStepBackward:
#ifdef DEBUG
      Serial.println("Step Backward");
#endif
      //commented - because can be serial cmd like cmdMove_froward
      //if (IsNotStepCommand(lastCmd))
      {
        StepMove(command);
        replyEnabled = false;
      }
      break;
    case cmdMoveStepTankRotateClockwise:
#ifdef DEBUG
      Serial.println("Step TankRotate ClockWise");
#endif
      if (IsNotStepCommand(lastCmd))
      {
        StepMove(command);
        replyEnabled = false;
      }
      break;
    case cmdMoveStepTankRotateAntiClockwise:
#ifdef DEBUG
      Serial.println("Step TankRotate AntiClockWise");
#endif
      if (IsNotStepCommand(lastCmd))
      {
        StepMove(command);
        replyEnabled = false;
      }
      break;
    case cmdMove_ForwardWithTelemetry:// Move Forward
#ifdef DEBUG
      Serial.println(F("MOVE FORWARD"));
#endif
      MoveForward();
      replyEnabled = true;
      break;
    case cmdMove_Forward:// Move Forward
#ifdef DEBUG
      Serial.println(F("MOVE FORWARD"));
#endif
      MoveForward();
      replyEnabled = false;
      break;
    case cmdMove_Backward:// Move Back
#ifdef DEBUG
      Serial.println(F("MOVE_BACKWARD"));
#endif
      MoveBackward();
      replyEnabled = false;
      break;
    case cmdMove_BackwardWithTelemetry:// Move Back
#ifdef DEBUG
      Serial.println(F("MOVE_BACKWARD"));
#endif
      MoveBackward();
      replyEnabled = true;
      break;
    case cmdMove_RotateAntiClockWise:
    case cmdMove_TankRotateAntiClockWise:
#ifdef DEBUG
      Serial.println(F("TANK ROTATE_ANTICLOCKWISE"));
#endif
      TankRotate(true);
      replyEnabled = false;
      break;
    case cmdMove_RotateClockWise:
    case cmdMove_TankRotateClockWise:
#ifdef DEBUG
      Serial.println(F("TANK ROTATE_CLOCKWISE"));
#endif
      TankRotate(false);
      replyEnabled = false;
      break;
    case cmdStopEngines:
      StopEngines();
      replyEnabled = true;
#ifdef DEBUG
      Serial.println(F("STOP ENGINES"));
#endif
      break;
  }
  //if that's ServoPos command, we need to process it separately (Roboarm section)
  if (command >= cmdSetHomeServoPos && command <= cmdServoPos36)
  {
#ifdef DEBUG
    Serial.println("ServoPosCommand");
#endif
    ServoPosCommand(command);
  }
  else if ((command >= cmdServoHRotationUp && command <= cmdMoveDown) ||
           (command >= cmdArmExpand && command <= cmdAutoCaptureBeacon))
  {
#ifdef DEBUG
    Serial.println("RoboarmCommand");
#endif
    RoboArmCommand(command);
  }
  else
  {
    switch (command)
    {
      //      case cmdDiagnostics:
      //#ifdef DEBUG
      //        Serial.println("cmdDiagnostics");
      //#endif
      //        break;
      case cmdLamp:
#ifdef DEBUG
        Serial.println("Enter cmdLamp");
#endif
        //Lamp(DiagDataBag.LampMode == false);
        Lamp(!IsLampOn());
#ifdef DEBUG
        Serial.println("cmdLamp");
#endif
        break;
      case cmdCamera1:
        SwitchCam(camCamera1);
#ifdef DEBUG
        Serial.println("cmdCamera1");
#endif
        break;
      case cmdCamera2:
        SwitchCam(camCamera2);
#ifdef DEBUG
        Serial.println("cmdCamera2");
#endif
        break;
      case cmdCamera3:
        SwitchCam(camCamera3);
#ifdef DEBUG
        Serial.println("cmdCamera3");
#endif
        break;
    }
  }
}

